unit mcOS_Fix;
{
  MicroCline Component to make programs compatile with Vista

  This component detects the operating system (sort of) and changes
  the form settings so that the application works

  Written by Robert Clemenzi 07-2011 (c)
  07-14-11 Start of file

  07-30-11 8 testcases created and passed
            mcVistaSucks\Help\Component_Test.html

  It was desired that this be automatic - changing the settings
  only if Vista (or above) was detected

  To force saving the ClientHeight and ClientWidth,
    form.AutoScroll must be false at *design time*
  Unfortunately, there is no way to know if the designer changes it
  Could try using the property getter

  form.Scaled can be changed in *create* but not in *loaded*
  unfortunately, none of this component's properties are
  set when *create* is called
  Therefore, I set it when the *AllowVistaScalling* property is set
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TmcVistaSucks = class(TComponent)
  private
    { Private declarations }
    FForm       : TForm;
    FScaled     : boolean;
    FAutoScroll : boolean;
    FAllowVistaScalling: boolean;
    procedure SetAllowVistaScalling(const Value: boolean);
    function GetAutoScroll: boolean;
  protected
    { Protected declarations }
    procedure Loaded; override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;
  published
    { Published declarations }
    property AllowVistaScaling : boolean  read FAllowVistaScalling write SetAllowVistaScalling;
    property AutoScroll : boolean  read GetAutoScroll write FAutoScroll;
  end;

procedure mcVistaSucks;
procedure Register;

function IsVista: Boolean;

implementation


var
  FmcVistaSucks: TmcVistaSucks;

procedure mcVistaSucks;
begin
  if FmcVistaSucks = nil then begin
   // FmcVistaSucks := TmcVistaSucks.Create(Application.MainForm);
  end;
end;

// The following is from IsWindowsVista in uvista.pas
// from "Making Delphi programs Vista-Ready"
function IsVista: Boolean;
var
  VerInfo: TOSVersioninfo;
begin
  VerInfo.dwOSVersionInfoSize := SizeOf(TOSVersionInfo);
  GetVersionEx(VerInfo);        
  Result := VerInfo.dwMajorVersion >= 6;
end;  

constructor TmcVistaSucks.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  // must be here for design time code in *Loaded*
  FForm := AOwner as TForm;

  if not (csDesigning in ComponentState) and IsVista then begin
{
    // debug has verified that the form properties are set at this time
    FScaled     := FForm.Scaled;       // default = true
    FAutoScroll := FForm.AutoScroll;   // default = true

    // however, the component properties are not known yet
    if not AllowVistaScalling then begin // not known yet
     // FForm.Scaled := false;
    end;
}
     // FForm.Scaled is already set at this point - checked via debug
     // Apparently, the form's properties have been set
     // Only *false* is stored in the dfm because *true* is the default
    FScaled      := FForm.Scaled; // default = true
    FForm.Scaled := false;

    FForm.AutoScroll := false; // should not be needed
  end;

end;   // TmcVistaSucks.Create

destructor TmcVistaSucks.Destroy;
begin
  inherited Destroy;
end;

procedure TmcVistaSucks.SetAllowVistaScalling(const Value: boolean);
begin
  // this is where the form value can be set
  // it is after this property is read and before the form is rendered
  FAllowVistaScalling := Value;

  if not (csDesigning in ComponentState) and IsVista then begin
     // used by Fixed_TTT.exe and Fixed_TFT.exe
    if FAllowVistaScalling then FForm.Scaled := FScaled;
  end;

end;

function TmcVistaSucks.GetAutoScroll: boolean;
begin
  // This tests the form and forces FForm.AutoScroll to false
  // It is not a perfect fix, but it seems ok
  
  if FForm.AutoScroll then begin
    FAutoScroll      := true;
    FForm.AutoScroll := false;
  end;

  Result := FAutoScroll;
end;

  // This runs after all parameters from the form file are loaded
  // Thus, if the form designer set some property, those are the values used
procedure TmcVistaSucks.Loaded;
begin
  inherited;

  if (csDesigning in ComponentState) then begin
    // Must set AutoScroll false at design time
    // to force saving the ClientHeight and ClientWidth

    if FForm.AutoScroll then begin
      FAutoScroll      := true;
      FForm.AutoScroll := false;
    end;

    exit;
  end;

  // required for XP and Vista
  // since FForm.AutoScroll must be false at design time
  FForm.AutoScroll := FAutoScroll ;

  if not IsVista then exit;

  // Decide which font to use
  // Added *FForm.Scaled* for Fixed_FTT.exe and Fixed_FFT.exe
  if AllowVistaScaling and FForm.Scaled then begin
    // uses Segoe UI 9 as the default font
   // FForm.Scaled := FScaled; // too late to have an effect
  end else begin
    // don't use the Vista font substitution
    if SameText (FForm.Font.Name, 'ms sans serif') then begin
      FForm.Font.Name := 'microsoft sans serif';
    end;
  end;

end;   // TmcVistaSucks.Loaded;

// **************************************************

procedure Register;
begin
  RegisterComponents('mc', [TmcVistaSucks]);
end;


initialization
//  At this time, Application.MainForm = nil
//  because this is called before Application.Initialize
  mcVistaSucks;

finalization
  FmcVistaSucks.Free;


end.
 
